@@ -407,6 +407,12 @@ is_path_hidden (const gchar *path)
 	return results;
 }
 
+gboolean 
+is_path_in_mount_folder (const gchar *path)
+{
+	return (g_strstr_len (path, strlen ("/mnt/"), "/mnt/") != NULL);
+}
+
 gboolean 
 is_path_in_home_folder (const gchar *path)
 {
