@@ -28,13 +28,13 @@
 #ifndef _GSEARCHTOOL_SUPPORT_H_
 #define _GSEARCHTOOL_SUPPORT_H_
 
-#include <gnome.h>
-
 #ifdef __cplusplus
 extern "C" {
 #pragma }
 #endif
 
+#include <gnome.h>
+
 #define CDDB_DISCLOSURE_TYPE (cddb_disclosure_get_type ())
 #define CDDB_DISCLOSURE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), CDDB_DISCLOSURE_TYPE, CDDBDisclosure))
 #define CDDB_DISCLOSURE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), CDDB_DISCLOSURE_TYPE, CDDBDisclosureClass))
@@ -61,12 +61,18 @@ cddb_disclosure_get_type 	(void);
 GtkWidget *
 cddb_disclosure_new 		(const char *shown,
 				 const char *hidden);
+void
+cddb_disclosure_set_container 	(CDDBDisclosure *disclosure,
+				 GtkWidget *container);
 gboolean  	
 is_path_hidden 			(const gchar *path);
 
 gboolean  	
 is_path_in_home_folder 		(const gchar *path);
 
+gboolean  	
+is_path_in_mount_folder		(const gchar *path);
+
 gboolean
 file_extension_is 		(const char *filename, 
 		   		 const char *ext);
